@@ -1,6 +1,6 @@
 ///////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2008. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -18,15 +18,18 @@
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
 
+#include <boost/pointer_to_other.hpp>
+
 #include <boost/interprocess/interprocess_fwd.hpp>
-#include <boost/interprocess/allocators/allocation_type.hpp>
+#include <boost/interprocess/containers/allocation_type.hpp>
+#include <boost/container/detail/multiallocation_chain.hpp>
+#include <boost/interprocess/allocators/detail/allocator_common.hpp>
 #include <boost/interprocess/detail/utilities.hpp>
-#include <boost/interprocess/detail/version_type.hpp>
+#include <boost/interprocess/containers/version_type.hpp>
 #include <boost/interprocess/exceptions.hpp>
 #include <boost/assert.hpp>
 #include <boost/utility/addressof.hpp>
 #include <boost/interprocess/detail/type_traits.hpp>
-#include <boost/interprocess/detail/iterators.hpp>
 
 #include <memory>
 #include <algorithm>
@@ -64,11 +67,11 @@ class allocator
 
    //Typedef to const void pointer
    typedef typename 
-      detail::pointer_to_other
+      boost::pointer_to_other
          <aux_pointer_t, const void>::type   cvoid_ptr;
 
    //Pointer to the allocator
-   typedef typename detail::pointer_to_other
+   typedef typename boost::pointer_to_other
       <cvoid_ptr, segment_manager>::type     alloc_ptr_t;
 
    //Not assignable from related allocator
@@ -84,31 +87,24 @@ class allocator
 
    public:
    typedef T                                    value_type;
-   typedef typename detail::pointer_to_other
+   typedef typename boost::pointer_to_other
       <cvoid_ptr, T>::type                      pointer;
-   typedef typename detail::
+   typedef typename boost::
       pointer_to_other<pointer, const T>::type  const_pointer;
-   typedef typename detail::add_reference
+   typedef typename ipcdetail::add_reference
                      <value_type>::type         reference;
-   typedef typename detail::add_reference
+   typedef typename ipcdetail::add_reference
                      <const value_type>::type   const_reference;
-   typedef std::size_t                          size_type;
-   typedef std::ptrdiff_t                       difference_type;
+   typedef typename segment_manager::size_type               size_type;
+   typedef typename segment_manager::difference_type         difference_type;
 
-   typedef detail::version_type<allocator, 2>   version;
+   typedef boost::interprocess::version_type<allocator, 2>   version;
 
    /// @cond
 
    //Experimental. Don't use.
-   typedef transform_iterator
-      < typename SegmentManager::
-         multiallocation_iterator
-      , detail::cast_functor <T> >              multiallocation_iterator;
-   typedef detail::multiallocation_chain_adaptor
-      <typename SegmentManager::
-         multiallocation_chain
-      , T>                                      multiallocation_chain;
-
+   typedef boost::container::containers_detail::transform_multiallocation_chain
+      <typename SegmentManager::multiallocation_chain, T>multiallocation_chain;
    /// @endcond
 
    //!Obtains an allocator that allocates
@@ -122,7 +118,7 @@ class allocator
    //!Returns the segment manager.
    //!Never throws
    segment_manager* get_segment_manager()const
-   {  return detail::get_pointer(mp_mngr);   }
+   {  return ipcdetail::get_pointer(mp_mngr);   }
 
    //!Constructor from the segment manager.
    //!Never throws
@@ -153,7 +149,7 @@ class allocator
    //!Deallocates memory previously allocated.
    //!Never throws
    void deallocate(const pointer &ptr, size_type)
-   {  mp_mngr->deallocate((void*)detail::get_pointer(ptr));  }
+   {  mp_mngr->deallocate((void*)ipcdetail::get_pointer(ptr));  }
 
    //!Returns the number of elements that could be allocated.
    //!Never throws
@@ -163,24 +159,24 @@ class allocator
    //!Swap segment manager. Does not throw. If each allocator is placed in
    //!different memory segments, the result is undefined.
    friend void swap(self_t &alloc1, self_t &alloc2)
-   {  detail::do_swap(alloc1.mp_mngr, alloc2.mp_mngr);   }
+   {  ipcdetail::do_swap(alloc1.mp_mngr, alloc2.mp_mngr);   }
 
    //!Returns maximum the number of objects the previously allocated memory
    //!pointed by p can hold. This size only works for memory allocated with
    //!allocate, allocation_command and allocate_many.
    size_type size(const pointer &p) const
    {  
-      return (size_type)mp_mngr->size(detail::get_pointer(p))/sizeof(T);
+      return (size_type)mp_mngr->size(ipcdetail::get_pointer(p))/sizeof(T);
    }
 
    std::pair<pointer, bool>
-      allocation_command(allocation_type command,
+      allocation_command(boost::interprocess::allocation_type command,
                          size_type limit_size, 
                          size_type preferred_size,
                          size_type &received_size, const pointer &reuse = 0)
    {
       return mp_mngr->allocation_command
-         (command, limit_size, preferred_size, received_size, detail::get_pointer(reuse));
+         (command, limit_size, preferred_size, received_size, ipcdetail::get_pointer(reuse));
    }
 
    //!Allocates many elements of size elem_size in a contiguous block
@@ -189,19 +185,19 @@ class allocator
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. The elements must be deallocated
    //!with deallocate(...)
-   multiallocation_iterator allocate_many(size_type elem_size, std::size_t num_elements)
+   multiallocation_chain allocate_many
+      (size_type elem_size, size_type num_elements)
    {
-      return multiallocation_iterator
-         (mp_mngr->allocate_many(sizeof(T)*elem_size, num_elements));
+      return multiallocation_chain(mp_mngr->allocate_many(sizeof(T)*elem_size, num_elements));
    }
 
    //!Allocates n_elements elements, each one of size elem_sizes[i]in a
    //!contiguous block
    //!of memory. The elements must be deallocated
-   multiallocation_iterator allocate_many(const size_type *elem_sizes, size_type n_elements)
+   multiallocation_chain allocate_many
+      (const size_type *elem_sizes, size_type n_elements)
    {
-      return multiallocation_iterator
-         (mp_mngr->allocate_many(elem_sizes, n_elements, sizeof(T)));
+      multiallocation_chain(mp_mngr->allocate_many(elem_sizes, n_elements, sizeof(T)));
    }
 
    //!Allocates many elements of size elem_size in a contiguous block
@@ -210,8 +206,10 @@ class allocator
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. The elements must be deallocated
    //!with deallocate(...)
-   void deallocate_many(multiallocation_iterator it)
-   {  return mp_mngr->deallocate_many(it.base()); }
+   void deallocate_many(multiallocation_chain chain)
+   {
+      return mp_mngr->deallocate_many(chain.extract_multiallocation_chain());
+   }
 
    //!Allocates just one object. Memory allocated with this function
    //!must be deallocated only with deallocate_one().
@@ -225,7 +223,8 @@ class allocator
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. Memory allocated with this function
    //!must be deallocated only with deallocate_one().
-   multiallocation_iterator allocate_individual(std::size_t num_elements)
+   multiallocation_chain allocate_individual
+      (size_type num_elements)
    {  return this->allocate_many(1, num_elements); }
 
    //!Deallocates memory previously allocated with allocate_one().
@@ -240,8 +239,8 @@ class allocator
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. Memory allocated with this function
    //!must be deallocated only with deallocate_one().
-   void deallocate_individual(multiallocation_iterator it)
-   {  return this->deallocate_many(it); }
+   void deallocate_individual(multiallocation_chain chain)
+   {  return this->deallocate_many(boost::interprocess::move(chain)); }
 
    //!Returns address of mutable object.
    //!Never throws
@@ -256,12 +255,12 @@ class allocator
    //!Copy construct an object
    //!Throws if T's copy constructor throws
    void construct(const pointer &ptr, const_reference v)
-   {  new((void*)detail::get_pointer(ptr)) value_type(v);  }
+   {  new((void*)ipcdetail::get_pointer(ptr)) value_type(v);  }
 
    //!Default construct an object. 
    //!Throws if T's default constructor throws
    void construct(const pointer &ptr)
-   {  new((void*)detail::get_pointer(ptr)) value_type;  }
+   {  new((void*)ipcdetail::get_pointer(ptr)) value_type;  }
 
    //!Destroys object. Throws if object's
    //!destructor throws
