@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2005-2008. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2005-2009. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -18,10 +18,13 @@
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
 
+#include <boost/pointer_to_other.hpp>
+
 #include <boost/interprocess/interprocess_fwd.hpp>
 #include <boost/assert.hpp>
 #include <boost/utility/addressof.hpp>
 #include <boost/interprocess/allocators/detail/adaptive_node_pool.hpp>
+#include <boost/container/detail/multiallocation_chain.hpp>
 #include <boost/interprocess/exceptions.hpp>
 #include <boost/interprocess/detail/utilities.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
@@ -37,7 +40,7 @@ namespace interprocess {
 
 /// @cond
 
-namespace detail {
+namespace ipcdetail {
 
 template < unsigned int Version
          , class T
@@ -65,7 +68,7 @@ class private_adaptive_pool_base
    typedef private_adaptive_pool_base
       < Version, T, SegmentManager, NodesPerBlock
       , MaxFreeBlocks, OverheadPercent>                  self_t;
-   typedef detail::private_adaptive_node_pool
+   typedef ipcdetail::private_adaptive_node_pool
       <SegmentManager
       , sizeof_value<T>::value
       , NodesPerBlock
@@ -78,25 +81,21 @@ class private_adaptive_pool_base
    /// @endcond
 
    public:
-   typedef typename detail::
+   typedef typename boost::
       pointer_to_other<void_pointer, T>::type            pointer;
-   typedef typename detail::
+   typedef typename boost::
       pointer_to_other<void_pointer, const T>::type      const_pointer;
    typedef T                                             value_type;
-   typedef typename detail::add_reference
+   typedef typename ipcdetail::add_reference
                      <value_type>::type                  reference;
-   typedef typename detail::add_reference
+   typedef typename ipcdetail::add_reference
                      <const value_type>::type            const_reference;
-   typedef std::size_t                                   size_type;
-   typedef std::ptrdiff_t                                difference_type;
-   typedef detail::version_type
+   typedef typename segment_manager::size_type           size_type;
+   typedef typename segment_manager::size_type           difference_type;
+   typedef boost::interprocess::version_type
       <private_adaptive_pool_base, Version>              version;
-   typedef transform_iterator
-      < typename SegmentManager::
-         multiallocation_iterator
-      , detail::cast_functor <T> >              multiallocation_iterator;
-   typedef typename SegmentManager::
-      multiallocation_chain                     multiallocation_chain;
+   typedef boost::container::containers_detail::transform_multiallocation_chain
+      <typename SegmentManager::multiallocation_chain, T>multiallocation_chain;
 
    //!Obtains node_allocator from other node_allocator
    template<class T2>
@@ -111,7 +110,7 @@ class private_adaptive_pool_base
    template <int dummy>
    struct node_pool
    {
-      typedef detail::private_adaptive_node_pool
+      typedef ipcdetail::private_adaptive_node_pool
       <SegmentManager
       , sizeof_value<T>::value
       , NodesPerBlock
@@ -204,7 +203,7 @@ class private_adaptive_pool_v1
          >
 {
    public:
-   typedef detail::private_adaptive_pool_base
+   typedef ipcdetail::private_adaptive_pool_base
          < 1, T, SegmentManager, NodesPerBlock, MaxFreeBlocks, OverheadPercent> base_t;
 
    template<class T2>
@@ -224,7 +223,7 @@ class private_adaptive_pool_v1
    {}
 };
 
-}  //namespace detail {
+}  //namespace ipcdetail {
 
 /// @endcond
 
@@ -249,7 +248,7 @@ template < class T
          >
 class private_adaptive_pool
    /// @cond
-   :  public detail::private_adaptive_pool_base
+   :  public ipcdetail::private_adaptive_pool_base
          < 2
          , T
          , SegmentManager
@@ -261,10 +260,10 @@ class private_adaptive_pool
 {
 
    #ifndef BOOST_INTERPROCESS_DOXYGEN_INVOKED
-   typedef detail::private_adaptive_pool_base
+   typedef ipcdetail::private_adaptive_pool_base
          < 2, T, SegmentManager, NodesPerBlock, MaxFreeBlocks, OverheadPercent> base_t;
    public:
-   typedef detail::version_type<private_adaptive_pool, 2>   version;
+   typedef boost::interprocess::version_type<private_adaptive_pool, 2>   version;
 
    template<class T2>
    struct rebind
@@ -290,12 +289,12 @@ class private_adaptive_pool
    typedef implementation_defined::pointer               pointer;
    typedef implementation_defined::const_pointer         const_pointer;
    typedef T                                             value_type;
-   typedef typename detail::add_reference
+   typedef typename ipcdetail::add_reference
                      <value_type>::type                  reference;
-   typedef typename detail::add_reference
+   typedef typename ipcdetail::add_reference
                      <const value_type>::type            const_reference;
-   typedef std::size_t                                   size_type;
-   typedef std::ptrdiff_t                                difference_type;
+   typedef typename segment_manager::size_type           size_type;
+   typedef typename segment_manager::difference_type     difference_type;
 
    //!Obtains private_adaptive_pool from 
    //!private_adaptive_pool
@@ -388,7 +387,7 @@ class private_adaptive_pool
    size_type size(const pointer &p) const;
 
    std::pair<pointer, bool>
-      allocation_command(allocation_type command,
+      allocation_command(boost::interprocess::allocation_type command,
                          size_type limit_size, 
                          size_type preferred_size,
                          size_type &received_size, const pointer &reuse = 0);
@@ -399,12 +398,12 @@ class private_adaptive_pool
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. The elements must be deallocated
    //!with deallocate(...)
-   multiallocation_iterator allocate_many(size_type elem_size, std::size_t num_elements);
+   multiallocation_chain allocate_many(size_type elem_size, size_type num_elements);
 
    //!Allocates n_elements elements, each one of size elem_sizes[i]in a
    //!contiguous block
    //!of memory. The elements must be deallocated
-   multiallocation_iterator allocate_many(const size_type *elem_sizes, size_type n_elements);
+   multiallocation_chain allocate_many(const size_type *elem_sizes, size_type n_elements);
 
    //!Allocates many elements of size elem_size in a contiguous block
    //!of memory. The minimum number to be allocated is min_elements,
@@ -412,7 +411,7 @@ class private_adaptive_pool
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. The elements must be deallocated
    //!with deallocate(...)
-   void deallocate_many(multiallocation_iterator it);
+   void deallocate_many(multiallocation_chain chain);
 
    //!Allocates just one object. Memory allocated with this function
    //!must be deallocated only with deallocate_one().
@@ -425,7 +424,7 @@ class private_adaptive_pool
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. Memory allocated with this function
    //!must be deallocated only with deallocate_one().
-   multiallocation_iterator allocate_individual(std::size_t num_elements);
+   multiallocation_chain allocate_individual(size_type num_elements);
 
    //!Deallocates memory previously allocated with allocate_one().
    //!You should never use deallocate_one to deallocate memory allocated
@@ -438,7 +437,7 @@ class private_adaptive_pool
    //!preferred_elements. The number of actually allocated elements is
    //!will be assigned to received_size. Memory allocated with this function
    //!must be deallocated only with deallocate_one().
-   void deallocate_individual(multiallocation_iterator it);
+   void deallocate_individual(multiallocation_chain chain);
    #endif
 };
 
