@@ -38,7 +38,7 @@ namespace interprocess{
 /// @cond
 template <class T, class D> class unique_ptr;
 
-namespace detail {
+namespace ipcdetail {
 
 template <class T> struct unique_ptr_error;
 
@@ -48,7 +48,7 @@ struct unique_ptr_error<const unique_ptr<T, D> >
     typedef unique_ptr<T, D> type;
 };
 
-}  //namespace detail {
+}  //namespace ipcdetail {
 /// @endcond
 
 //!Template unique_ptr stores a pointer to an object and deletes that object
@@ -83,14 +83,15 @@ class unique_ptr
 {
    /// @cond
    struct nat {int for_bool_;};
-   typedef typename detail::add_reference<D>::type deleter_reference;
-   typedef typename detail::add_reference<const D>::type deleter_const_reference;
+   typedef typename ipcdetail::add_reference<D>::type deleter_reference;
+   typedef typename ipcdetail::add_reference<const D>::type deleter_const_reference;
    /// @endcond
 
    public:
+
    typedef T element_type;
    typedef D deleter_type;
-   typedef typename detail::pointer_type<T, D>::type pointer;
+   typedef typename ipcdetail::pointer_type<T, D>::type pointer;
 
    //!Requires: D must be default constructible, and that construction must not
    //!throw an exception. D must not be a reference type.
@@ -127,9 +128,9 @@ class unique_ptr
    //!
    //!Throws: nothing.
    unique_ptr(pointer p
-             ,typename detail::if_<detail::is_reference<D>
+             ,typename ipcdetail::if_<ipcdetail::is_reference<D>
                   ,D
-                  ,typename detail::add_reference<const D>::type>::type d)
+                  ,typename ipcdetail::add_reference<const D>::type>::type d)
       : ptr_(p, d)
    {}
 
@@ -138,11 +139,11 @@ class unique_ptr
    //!
    //!Effects: Constructs a unique_ptr which owns the pointer which u owns
    //!(if any). If the deleter is not a reference type, it is move constructed
-   //!from us deleter, otherwise the reference is copy constructed from us deleter.
+   //!from u's deleter, otherwise the reference is copy constructed from u's deleter.
    //!
    //!After the construction, u no longer owns a pointer.
    //![ Note: The deleter constructor can be implemented with
-   //!std::forward<D>. end note ]
+   //!   boost::interprocess::forward<D>. -end note ]
    //!
    //!Postconditions: get() == value u.get() had before the construction.
    //!get_deleter() returns a reference to the internally stored deleter which
@@ -150,15 +151,9 @@ class unique_ptr
    //!deleter() and u.get_deleter() both reference the same lvalue deleter.
    //!
    //!Throws: nothing.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   unique_ptr(detail::moved_object<unique_ptr> u)
-      : ptr_(u.get().release(), move(u.get().get_deleter()))
+   unique_ptr(BOOST_RV_REF(unique_ptr) u)
+      : ptr_(u.release(), boost::interprocess::forward<D>(u.get_deleter()))
    {}
-   #else
-   unique_ptr(unique_ptr &&u)
-      : ptr_(u.release(), forward<D>(u.get_deleter()))
-   {}
-   #endif
 
    //!Requires: If D is not a reference type, construction of the deleter
    //!D from an rvalue of type E must be well formed
@@ -168,8 +163,8 @@ class unique_ptr
    //!
    //!Effects: Constructs a unique_ptr which owns the pointer which u owns
    //!(if any). If the deleter is not a reference
-   //!type, it is move constructed from us deleter, otherwise the reference
-   //!is copy constructed from us deleter.
+   //!type, it is move constructed from u's deleter, otherwise the reference
+   //!is copy constructed from u's deleter.
    //!
    //!After the construction, u no longer owns a pointer.
    //!
@@ -179,37 +174,20 @@ class unique_ptr
    //!was constructed from u.get_deleter().
    //!
    //!Throws: nothing.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
    template <class U, class E>
-   unique_ptr(const detail::moved_object<unique_ptr<U, E> >& u,
-      typename detail::enable_if_c<
-            detail::is_convertible<typename unique_ptr<U, E>::pointer, pointer>::value &&
-            detail::is_convertible<E, D>::value &&
+   unique_ptr(BOOST_RV_REF_2_TEMPL_ARGS(unique_ptr, U, E) u,
+      typename ipcdetail::enable_if_c<
+            ipcdetail::is_convertible<typename unique_ptr<U, E>::pointer, pointer>::value &&
+            ipcdetail::is_convertible<E, D>::value &&
             (
-               !detail::is_reference<D>::value ||
-               detail::is_same<D, E>::value
+               !ipcdetail::is_reference<D>::value ||
+               ipcdetail::is_same<D, E>::value
             )
             ,
             nat
             >::type = nat())
-      : ptr_(const_cast<unique_ptr<U,E>&>(u.get()).release(), move(u.get().get_deleter()))
+      : ptr_(const_cast<unique_ptr<U,E>&>(u).release(), boost::interprocess::move<D>(u.get_deleter()))
    {}
-   #else
-   template <class U, class E>
-   unique_ptr(unique_ptr<U, E> && u,
-      typename boost::enable_if_c<
-            detail::is_convertible<typename unique_ptr<U, E>::pointer, pointer>::value &&
-            detail::is_convertible<E, D>::value &&
-            (
-               !detail::is_reference<D>::value ||
-               detail::is_same<D, E>::value
-            )
-            ,
-            nat
-            >::type = nat())
-      : ptr_(const_cast<unique_ptr<U,E>&>(u).release(), forward<D>(u.get_deleter()))
-   {}
-   #endif
 
    //!Effects: If get() == 0 there are no effects. Otherwise get_deleter()(get()).
    //!
@@ -221,7 +199,7 @@ class unique_ptr
 
    //!Requires: Assignment of the deleter D from an rvalue D must not throw an exception.
    //!
-   //!Effects: reset(u.release()) followed by a move assignment from us deleter to
+   //!Effects: reset(u.release()) followed by a move assignment from u's deleter to
    //!this deleter.
    //!
    //!Postconditions: This unique_ptr now owns the pointer which u owned, and u no
@@ -230,27 +208,18 @@ class unique_ptr
    //!Returns: *this.
    //!
    //!Throws: nothing.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   unique_ptr& operator=(const detail::moved_object<unique_ptr>& u)
-   {
-      reset(u.get().release());
-      ptr_.second() = move(u.get().get_deleter());
-      return *this;
-   }
-   #else
-   unique_ptr(unique_ptr && u)
+   unique_ptr& operator=(BOOST_RV_REF(unique_ptr) u)
    {
       reset(u.release());
-      ptr_.second() = move(u.get_deleter());
+      ptr_.second() = boost::interprocess::move(u.get_deleter());
       return *this;
    }
-   #endif
 
    //!Requires: Assignment of the deleter D from an rvalue D must not
    //!throw an exception. U* must be implicitly convertible to T*.
    //!
    //!Effects: reset(u.release()) followed by a move assignment from
-   //!us deleter to this deleter. If either D or E is
+   //!u's deleter to this deleter. If either D or E is
    //!a reference type, then the referenced lvalue deleter participates
    //!in the move assignment.
    //!
@@ -261,21 +230,12 @@ class unique_ptr
    //!
    //!Throws: nothing.
    template <class U, class E>
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
-   unique_ptr& operator=(const detail::moved_object<unique_ptr<U, E> >& mu)
-   {
-      reset(mu.get().release());
-      ptr_.second() = move(mu.get().get_deleter());
-      return *this;
-   }
-   #else
-   unique_ptr(unique_ptr<U, E> && u)
+   unique_ptr& operator=(BOOST_RV_REF_2_TEMPL_ARGS(unique_ptr, U, E) u)
    {
       reset(u.release());
-      ptr_.second() = move(u.get_deleter());
+      ptr_.second() = boost::interprocess::move(u.get_deleter());
       return *this;
    }
-   #endif
 
    //!Assigns from the literal 0 or NULL.
    //!
@@ -295,7 +255,7 @@ class unique_ptr
    //!Requires: get() != 0.
    //!Returns: *get().
    //!Throws: nothing.
-   typename detail::add_reference<T>::type operator*()  const
+   typename ipcdetail::add_reference<T>::type operator*()  const
    {  return *ptr_.first();   }
 
    //!Requires: get() != 0.
@@ -356,32 +316,21 @@ class unique_ptr
 
    //!Requires: The deleter D is Swappable and will not throw an exception under swap.
    //!
-   //!Effects: The stored pointers of this and u are exchanged. The stored deleters are swapd (unqualified).
-   //!
+   //!Effects: The stored pointers of this and u are exchanged.
+   //!   The stored deleters are swapped (unqualified).
    //!Throws: nothing.
-   #ifndef BOOST_INTERPROCESS_RVALUE_REFERENCE
    void swap(unique_ptr& u)
    {  ptr_.swap(u.ptr_);   }
 
-   void swap(detail::moved_object<unique_ptr> mu)
-   {  ptr_.swap(mu.get().ptr_);  }
-   #else
-   void swap(unique_ptr&&u)
-   {  ptr_.swap(u.ptr_);   }
-   #endif
-
    /// @cond
    private:
    boost::compressed_pair<pointer, D> ptr_;
-
-   //This private constructor avoids moving from non-const lvalues
-   unique_ptr(const unique_ptr&);
+   BOOST_MOVABLE_BUT_NOT_COPYABLE(unique_ptr)
    template <class U, class E> unique_ptr(unique_ptr<U, E>&);
-   template <class U> unique_ptr(U&, typename detail::unique_ptr_error<U>::type = 0);
+   template <class U> unique_ptr(U&, typename ipcdetail::unique_ptr_error<U>::type = 0);
    
-   unique_ptr& operator=(unique_ptr&);
    template <class U, class E> unique_ptr& operator=(unique_ptr<U, E>&);
-   template <class U> typename detail::unique_ptr_error<U>::type operator=(U&);
+   template <class U> typename ipcdetail::unique_ptr_error<U>::type operator=(U&);
    /// @endcond
 };
 /*
@@ -389,12 +338,12 @@ template <class T, class D>
 class unique_ptr<T[], D>
 {
     struct nat {int for_bool_;};
-    typedef typename detail::add_reference<D>::type deleter_reference;
-    typedef typename detail::add_reference<const D>::type deleter_const_reference;
+    typedef typename ipcdetail::add_reference<D>::type deleter_reference;
+    typedef typename ipcdetail::add_reference<const D>::type deleter_const_reference;
 public:
     typedef T element_type;
     typedef D deleter_type;
-    typedef typename detail::pointer_type<T, D>::type pointer;
+    typedef typename ipcdetail::pointer_type<T, D>::type pointer;
 
     // constructors
     unique_ptr() : ptr_(pointer()) {}
@@ -402,7 +351,7 @@ public:
     unique_ptr(pointer p, typename if_<
                           boost::is_reference<D>,
                           D,
-                          typename detail::add_reference<const D>::type>::type d)
+                          typename ipcdetail::add_reference<const D>::type>::type d)
         : ptr_(p, d) {}
     unique_ptr(const unique_ptr& u)
         : ptr_(const_cast<unique_ptr&>(u).release(), u.get_deleter()) {}
@@ -425,7 +374,7 @@ public:
     }
 
     // observers
-    typename detail::add_reference<T>::type operator[](std::size_t i)  const {return ptr_.first()[i];}
+    typename ipcdetail::add_reference<T>::type operator[](std::size_t i)  const {return ptr_.first()[i];}
     pointer get()        const {return ptr_.first();}
     deleter_reference       get_deleter()       {return ptr_.second();}
     deleter_const_reference get_deleter() const {return ptr_.second();}
@@ -457,22 +406,22 @@ private:
         typename boost::enable_if<boost::is_convertible<U, pointer> >::type* = 0);
 
     unique_ptr(unique_ptr&);
-    template <class U> unique_ptr(U&, typename detail::unique_ptr_error<U>::type = 0);
+    template <class U> unique_ptr(U&, typename ipcdetail::unique_ptr_error<U>::type = 0);
 
     unique_ptr& operator=(unique_ptr&);
-    template <class U> typename detail::unique_ptr_error<U>::type operator=(U&);
+    template <class U> typename ipcdetail::unique_ptr_error<U>::type operator=(U&);
 };
 
 template <class T, class D, std::size_t N>
 class unique_ptr<T[N], D>
 {
     struct nat {int for_bool_;};
-    typedef typename detail::add_reference<D>::type deleter_reference;
-    typedef typename detail::add_reference<const D>::type deleter_const_reference;
+    typedef typename ipcdetail::add_reference<D>::type deleter_reference;
+    typedef typename ipcdetail::add_reference<const D>::type deleter_const_reference;
 public:
     typedef T element_type;
     typedef D deleter_type;
-    typedef typename detail::pointer_type<T, D>::type pointer;
+    typedef typename ipcdetail::pointer_type<T, D>::type pointer;
     static const std::size_t size = N;
 
     // constructors
@@ -481,7 +430,7 @@ public:
     unique_ptr(pointer p, typename if_<
                          boost::is_reference<D>,
                          D,
-                         typename detail::add_reference<const D>::type>::type d)
+                         typename ipcdetail::add_reference<const D>::type>::type d)
         : ptr_(p, d) {}
     unique_ptr(const unique_ptr& u)
         : ptr_(const_cast<unique_ptr&>(u).release(), u.get_deleter()) {}
@@ -504,7 +453,7 @@ public:
     }
 
     // observers
-    typename detail::add_reference<T>::type operator[](std::size_t i)  const {return ptr_.first()[i];}
+    typename ipcdetail::add_reference<T>::type operator[](std::size_t i)  const {return ptr_.first()[i];}
     pointer get()        const {return ptr_.first();}
     deleter_reference       get_deleter()       {return ptr_.second();}
     deleter_const_reference get_deleter() const {return ptr_.second();}
@@ -536,10 +485,10 @@ private:
         typename boost::enable_if<boost::is_convertible<U, pointer> >::type* = 0);
 
     unique_ptr(unique_ptr&);
-    template <class U> unique_ptr(U&, typename detail::unique_ptr_error<U>::type = 0);
+    template <class U> unique_ptr(U&, typename ipcdetail::unique_ptr_error<U>::type = 0);
 
     unique_ptr& operator=(unique_ptr&);
-    template <class U> typename detail::unique_ptr_error<U>::type operator=(U&);
+    template <class U> typename ipcdetail::unique_ptr_error<U>::type operator=(U&);
 };
 */
 template <class T, class D> inline
@@ -570,16 +519,6 @@ template <class T1, class D1, class T2, class D2> inline
 bool operator>=(const unique_ptr<T1, D1>& x, const unique_ptr<T2, D2>& y)
 {  return x.get() >= y.get(); }
 
-/// @cond
-
-//!This class has move constructor
-template <class T, class D>
-struct is_movable<unique_ptr<T, D> >
-{
-   enum {   value = true };
-};
-/// @endcond
-
 
 //!Returns the type of a unique pointer
 //!of type T with boost::interprocess::deleter deleter
@@ -593,20 +532,15 @@ struct managed_unique_ptr
    > type;
 };
 
-//!Returns an instance of the a unique pointer constructed
+//!Returns an instance of a unique pointer constructed
 //!with boost::interproces::deleter from a pointer
 //!of type T that has been allocated in the passed managed segment
 template<class T, class ManagedMemory>
-inline typename detail::return_type
-   <typename managed_unique_ptr<T, ManagedMemory>::type
-   >::type 
+inline typename managed_unique_ptr<T, ManagedMemory>::type
    make_managed_unique_ptr(T *constructed_object, ManagedMemory &managed_memory)
 {
-   typename managed_unique_ptr<T, ManagedMemory>::type to_return
-   ( constructed_object
-   , managed_memory.template get_deleter<T>()
-   );
-   return to_return;
+   return typename managed_unique_ptr<T, ManagedMemory>::type 
+      (constructed_object, managed_memory.template get_deleter<T>());
 }
 
 }  //namespace interprocess{
