@@ -11,18 +11,6 @@
 // Parts of the pthread code come from Boost Threads code.
 //
 //////////////////////////////////////////////////////////////////////////////
-//
-// Copyright (C) 2001-2003
-// William E. Kempf
-//
-// Permission to use, copy, modify, distribute and sell this software
-// and its documentation for any purpose is hereby granted without fee,
-// provided that the above copyright notice appear in all copies and
-// that both that copyright notice and this permission notice appear
-// in supporting documentation.  William E. Kempf makes no representations
-// about the suitability of this software for any purpose.
-// It is provided "as is" without express or implied warranty.
-//////////////////////////////////////////////////////////////////////////////
 
 #ifndef BOOST_INTERPROCESS_MUTEX_HPP
 #define BOOST_INTERPROCESS_MUTEX_HPP
@@ -33,10 +21,11 @@
 #  pragma once
 #endif
 
+#include <boost/interprocess/exceptions.hpp>
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/detail/workaround.hpp>
 #include <boost/interprocess/detail/posix_time_types_wrk.hpp>
-#include <cassert>
+#include <boost/assert.hpp>
 
 #if !defined(BOOST_INTERPROCESS_FORCE_GENERIC_EMULATION) && defined (BOOST_INTERPROCESS_POSIX_PROCESS_SHARED)
    #include <pthread.h>
@@ -44,10 +33,19 @@
    #include <boost/interprocess/sync/posix/pthread_helpers.hpp>
    #define BOOST_INTERPROCESS_USE_POSIX
 #else
-   #include <boost/interprocess/detail/atomic.hpp>
-   #include <boost/cstdint.hpp>
-   #include <boost/interprocess/detail/os_thread_functions.hpp>
+   #include <boost/interprocess/sync/emulation/mutex.hpp>
    #define BOOST_INTERPROCESS_USE_GENERIC_EMULATION
+
+namespace boost {
+namespace interprocess {
+namespace ipcdetail{
+namespace robust_emulation_helpers {
+
+template<class T>
+class mutex_traits;
+
+}}}}
+
 #endif
 
 /// @endcond
@@ -111,7 +109,9 @@ class interprocess_mutex
    private:
 
    #if   defined(BOOST_INTERPROCESS_USE_GENERIC_EMULATION)
-      volatile boost::uint32_t m_s;
+   friend class ipcdetail::robust_emulation_helpers::mutex_traits<interprocess_mutex>;
+   void take_ownership(){ mutex.take_ownership(); }
+   ipcdetail::emulation_mutex mutex;
    #elif defined(BOOST_INTERPROCESS_USE_POSIX)
       pthread_mutex_t   m_mut;
    #endif   //#if (defined BOOST_INTERPROCESS_WINDOWS)
@@ -119,17 +119,42 @@ class interprocess_mutex
 };
 
 }  //namespace interprocess {
-
 }  //namespace boost {
 
 #ifdef BOOST_INTERPROCESS_USE_GENERIC_EMULATION
 #  undef BOOST_INTERPROCESS_USE_GENERIC_EMULATION
-#  include <boost/interprocess/sync/emulation/interprocess_mutex.hpp>
+
+namespace boost {
+namespace interprocess {
+
+inline interprocess_mutex::interprocess_mutex(){}
+inline interprocess_mutex::~interprocess_mutex(){}
+inline void interprocess_mutex::lock()
+{
+#ifdef BOOST_INTERPROCESS_ENABLE_TIMEOUT_WHEN_LOCKING
+   boost::posix_time::ptime wait_time
+      = boost::posix_time::microsec_clock::universal_time()
+        + boost::posix_time::milliseconds(BOOST_INTERPROCESS_TIMEOUT_WHEN_LOCKING_DURATION_MS);
+   if (!mutex.timed_lock(wait_time))
+   {
+      throw interprocess_exception(timeout_when_locking_error, "Interprocess mutex timeout when locking. Possible deadlock: owner died without unlocking?");
+   }
+#else
+   mutex.lock();
+#endif
+}
+inline bool interprocess_mutex::try_lock(){ return mutex.try_lock(); }
+inline bool interprocess_mutex::timed_lock(const boost::posix_time::ptime &abs_time){ return mutex.timed_lock(abs_time); }
+inline void interprocess_mutex::unlock(){ mutex.unlock(); }
+
+}  //namespace interprocess {
+}  //namespace boost {
+
 #endif
 
 #ifdef BOOST_INTERPROCESS_USE_POSIX
+#include <boost/interprocess/sync/posix/interprocess_mutex.hpp>
 #  undef BOOST_INTERPROCESS_USE_POSIX
-#  include <boost/interprocess/sync/posix/interprocess_mutex.hpp>
 #endif
 
 #include <boost/interprocess/detail/config_end.hpp>
