@@ -1,5 +1,5 @@
-//  Copyright (c) 2001-2010 Hartmut Kaiser
-//  Copyright (c) 2001-2010 Joel de Guzman
+//  Copyright (c) 2001-2011 Hartmut Kaiser
+//  Copyright (c) 2001-2011 Joel de Guzman
 //
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -20,9 +20,13 @@
 #include <boost/spirit/home/support/info.hpp>
 #include <boost/spirit/home/support/detail/what_function.hpp>
 #include <boost/spirit/home/karma/detail/attributes.hpp>
+#include <boost/spirit/home/karma/detail/indirect_iterator.hpp>
 #include <boost/spirit/home/support/algorithm/any_if.hpp>
 #include <boost/spirit/home/support/unused.hpp>
 #include <boost/spirit/home/support/sequence_base_id.hpp>
+#include <boost/spirit/home/support/has_semantic_action.hpp>
+#include <boost/spirit/home/support/handles_container.hpp>
+#include <boost/spirit/home/support/attributes.hpp>
 #include <boost/fusion/include/vector.hpp>
 #include <boost/fusion/include/as_vector.hpp>
 #include <boost/fusion/include/for_each.hpp>
@@ -80,109 +84,11 @@ namespace boost { namespace spirit { namespace traits
           , mpl::bitor_<mpl::_2, mpl::_1>
         >::type type;
     };
-
 }}}
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace boost { namespace spirit { namespace karma
 {
-    namespace detail
-    {
-        template <typename T>
-        struct attribute_size
-          : fusion::result_of::size<T>
-        {};
-
-        template <>
-        struct attribute_size<unused_type>
-          : mpl::int_<0>
-        {};
-
-        template <typename Attribute>
-        inline typename enable_if<
-            mpl::and_<
-                fusion::traits::is_sequence<Attribute>
-              , mpl::not_<traits::is_container<Attribute> > >
-          , std::size_t
-        >::type
-        attr_size(Attribute const& attr)
-        {
-            return fusion::size(attr);
-        }
-
-        template <typename Attribute>
-        inline typename enable_if<
-            traits::is_container<Attribute>, std::size_t
-        >::type
-        attr_size(Attribute const& attr)
-        {
-            return attr.size();
-        }
-
-        inline std::size_t attr_size(unused_type)
-        {
-            return 0;
-        }
-
-        ///////////////////////////////////////////////////////////////////////
-        // This is a wrapper for any iterator allowing to pass a reference of it
-        // to the components of the sequence
-        template <typename Iterator>
-        class indirect_iterator
-          : public boost::iterator_facade<
-                indirect_iterator<Iterator>
-              , typename boost::detail::iterator_traits<Iterator>::value_type
-              , boost::forward_traversal_tag
-              , typename boost::detail::iterator_traits<Iterator>::value_type const&>
-        {
-            typedef typename boost::detail::iterator_traits<Iterator>::value_type
-                base_value_type;
-
-            typedef boost::iterator_facade<
-                indirect_iterator<Iterator>, base_value_type
-              , boost::forward_traversal_tag, base_value_type const&
-            > base_type;
-
-        public:
-            indirect_iterator(Iterator& iter)
-              : iter_(&iter)
-            {}
-
-        private:
-            friend class boost::iterator_core_access;
-
-            void increment()
-            {
-                ++*iter_;
-            }
-
-            bool equal(indirect_iterator const& other) const
-            {
-                return *iter_ == *other.iter_;
-            }
-
-            typename base_type::reference dereference() const
-            {
-                return **iter_;
-            }
-
-        private:
-            Iterator* iter_;
-        };
-
-        template <typename Iterator>
-        struct make_indirect_iterator
-        {
-            typedef indirect_iterator<Iterator> type;
-        };
-
-        template <>
-        struct make_indirect_iterator<unused_type const*>
-        {
-            typedef unused_type const* type;
-        };
-    }
-
     template <typename Elements, typename Strict, typename Derived>
     struct base_sequence : nary_generator<Derived>
     {
@@ -199,8 +105,9 @@ namespace boost { namespace spirit { namespace karma
         {
             // Put all the element attributes in a tuple
             typedef typename traits::build_attribute_sequence<
-                Elements, Context, mpl::identity, Iterator>::type
-            all_attributes;
+                Elements, Context, traits::sequence_attribute_transform
+              , Iterator, karma::domain
+            >::type all_attributes;
 
             // Now, build a fusion vector over the attributes. Note
             // that build_fusion_vector 1) removes all unused attributes
@@ -242,6 +149,8 @@ namespace boost { namespace spirit { namespace karma
             bool r = spirit::any_if(elements, attr
                           , fail_function(sink, ctx, d), predicate());
 
+            typedef typename traits::attribute_size<Attribute>::type size_type;
+
             // fail generating if sequences have not the same (logical) length
             return !r && (!Strict::value || 
                 // This ignores container element count (which is not good), 
@@ -252,7 +161,7 @@ namespace boost { namespace spirit { namespace karma
                 // is not optimal but much better than letting _all_ repetitive
                 // components fail.
                 Pred1::value ||
-                detail::attribute_size<attr_type_>::value == detail::attr_size(attr_));
+                size_type(traits::sequence_size<attr_type_>::value) == traits::size(attr_));
         }
 
         // Special case when Attribute is an stl container and the sequence's
@@ -268,12 +177,15 @@ namespace boost { namespace spirit { namespace karma
             typedef detail::fail_function<
                 OutputIterator, Context, Delimiter> fail_function;
 
-            typedef typename traits::container_iterator<Attribute const>::type 
-                iterator_type;
-            typedef typename detail::make_indirect_iterator<iterator_type>::type 
-                indirect_iterator_type;
+            typedef typename traits::container_iterator<
+                typename add_const<Attribute>::type
+            >::type iterator_type;
+
+            typedef 
+                typename traits::make_indirect_iterator<iterator_type>::type 
+            indirect_iterator_type;
             typedef detail::pass_container<
-                fail_function, Attribute, indirect_iterator_type, Strict>
+              fail_function, Attribute, indirect_iterator_type, mpl::true_>
             pass_container;
 
             iterator_type begin = traits::begin(attr_);
@@ -359,10 +271,20 @@ namespace boost { namespace spirit { namespace karma
     struct make_composite<proto::tag::shift_left, Elements, Modifiers>
       : detail::make_sequence<Elements, detail::get_stricttag<Modifiers>::value>
     {};
+
+    ///////////////////////////////////////////////////////////////////////////
+    // Helper template allowing to get the required container type for a rule
+    // attribute, which is part of a sequence.
+    template <typename Iterator>
+    struct make_sequence_iterator_range
+    {
+        typedef iterator_range<detail::indirect_iterator<Iterator> > type;
+    };
 }}} 
 
 namespace boost { namespace spirit { namespace traits
 {
+    ///////////////////////////////////////////////////////////////////////////
     template <typename Elements>
     struct has_semantic_action<karma::sequence<Elements> >
       : nary_has_semantic_action<Elements> {};
@@ -370,6 +292,19 @@ namespace boost { namespace spirit { namespace traits
     template <typename Elements>
     struct has_semantic_action<karma::strict_sequence<Elements> >
       : nary_has_semantic_action<Elements> {};
+
+    ///////////////////////////////////////////////////////////////////////////
+    template <typename Elements, typename Attribute, typename Context
+      , typename Iterator>
+    struct handles_container<karma::sequence<Elements>, Attribute, Context
+          , Iterator>
+      : mpl::true_ {};
+
+    template <typename Elements, typename Attribute, typename Context
+      , typename Iterator>
+    struct handles_container<karma::strict_sequence<Elements>, Attribute
+          , Context, Iterator>
+      : mpl::true_ {};
 }}}
 
 #endif
