@@ -5,7 +5,7 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2009 Hartmut Kaiser. Distributed under the Boost
+    Copyright (c) 2001-2011 Hartmut Kaiser. Distributed under the Boost
     Software License, Version 1.0. (See accompanying file
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -140,6 +140,7 @@ public:
     void reset_macromap();
 
     position_type &get_main_pos() { return main_pos; }
+    position_type const& get_main_pos() const { return main_pos; }
 
 //  interface for macro name introspection
     typedef typename defined_macros_type::name_iterator name_iterator;
@@ -421,8 +422,9 @@ token_id id = token_id(*begin);
         !IS_EXTCATEGORY(id, OperatorTokenType|AltExtTokenType) &&
         !IS_CATEGORY(id, BoolLiteralTokenType)) 
     {
+        std::string msg(impl::get_full_name(begin, end));
         BOOST_WAVE_THROW_CTX(ctx, preprocess_exception, invalid_macroname, 
-            impl::get_full_name(begin, end).c_str(), main_pos);
+            msg.c_str(), main_pos);
         return false;
     }
 
@@ -432,8 +434,9 @@ typename defined_macros_type::iterator cit;
 
     if (++it != end) {
     // there should be only one token as the inspected name
+        std::string msg(impl::get_full_name(begin, end));
         BOOST_WAVE_THROW_CTX(ctx, preprocess_exception, invalid_macroname, 
-            impl::get_full_name(begin, end).c_str(), main_pos);
+            msg.c_str(), main_pos);
         return false;
     }
     return is_defined(name, cit, 0);
@@ -572,7 +575,7 @@ macromap<ContextT>::expand_tokensequence_worker(
 // return the next one from there
     if (!pending.empty()) {
     on_exit::pop_front<definition_container_type> pop_front_token(pending);
-    
+
         return act_token = pending.front();
     }
 
@@ -580,20 +583,20 @@ macromap<ContextT>::expand_tokensequence_worker(
 //  T_IDENTIFIER token, try to replace this as a macro etc.
     using namespace boost::wave;
     typedef unput_queue_iterator<IteratorT, token_type, ContainerT> iterator_type;
-    
+
     if (first != last) {
     token_id id = token_id(*first);
 
     // ignore placeholder tokens
         if (T_PLACEHOLDER == id) {
         token_type placeholder = *first;
-        
+
             ++first;
             if (first == last)
                 return act_token = placeholder;
             id = token_id(*first);
         }
-            
+
         if (T_IDENTIFIER == id || IS_CATEGORY(id, KeywordTokenType) ||
             IS_EXTCATEGORY(id, OperatorTokenType|AltExtTokenType) ||
             IS_CATEGORY(id, BoolLiteralTokenType)) 
@@ -608,7 +611,7 @@ macromap<ContextT>::expand_tokensequence_worker(
             {
             // in C99 mode only: resolve the operator _Pragma
             token_type curr_token = *first;
-            
+
                 if (!resolve_operator_pragma(first, last, pending, seen_newline) ||
                     pending.size() > 0) 
                 {
@@ -618,7 +621,7 @@ macromap<ContextT>::expand_tokensequence_worker(
 
                     return act_token = pending.front();
                 }
-                
+
             // the operator _Pragma() was eaten completely, continue
                 return act_token = token_type(T_PLACEHOLDER, "_", 
                     curr_token.get_position());
@@ -626,7 +629,7 @@ macromap<ContextT>::expand_tokensequence_worker(
 
         token_type name_token (*first);
         typename defined_macros_type::iterator it;
-        
+
             if (is_defined(name_token.get_value(), it)) {
             // the current token contains an identifier, which is currently 
             // defined as a macro
@@ -638,12 +641,12 @@ macromap<ContextT>::expand_tokensequence_worker(
                     if (first != last) {
                     // splice the last token back into the input queue
                     typename ContainerT::reverse_iterator rit = pending.rbegin();
-                    
+
                         first.get_unput_queue().splice(
                             first.get_unput_queue().begin(), pending,
                             (++rit).base(), pending.end());
                     }
-                                            
+
                 // fall through ...
                 }
                 else if (!pending.empty()) {
@@ -717,7 +720,7 @@ macromap<ContextT>::collect_arguments (token_type const curr_token,
     using namespace boost::wave;
 
     arguments.push_back(ContainerT());
-    
+
 // collect the actual arguments
 typename std::vector<ContainerT>::size_type count_arguments = 0;
 int nested_parenthesis_level = 1;
@@ -738,14 +741,14 @@ token_type startof_argument_list = *next;
                 main_pos);
             return 0;
         }
-        
+
         switch (static_cast<unsigned int>(id)) {
         case T_LEFTPAREN:
             ++nested_parenthesis_level;
             argument->push_back(*next);
             was_whitespace = false;
             break;
-            
+
         case T_RIGHTPAREN:
             {
                 if (--nested_parenthesis_level >= 1)
@@ -777,7 +780,7 @@ token_type startof_argument_list = *next;
                 was_whitespace = false;
             }
             break;
-        
+
         case T_COMMA:
             if (1 == nested_parenthesis_level) {
             // next parameter
@@ -820,7 +823,7 @@ token_type startof_argument_list = *next;
 
         case T_PLACEHOLDER:
             break;      // ignore placeholder
-                        
+
         default:
             argument->push_back(*next);
             was_whitespace = false;
@@ -842,7 +845,7 @@ token_type startof_argument_list = *next;
         arguments.clear();
     }
     return count_arguments;
-}        
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // 
@@ -900,17 +903,17 @@ macromap<ContextT>::expand_argument (
     // expand the argument only once
         typedef typename std::vector<ContainerT>::value_type::iterator 
             argument_iterator_type;
-            
+
         argument_iterator_type begin_it = arguments[arg].begin();
         argument_iterator_type end_it = arguments[arg].end();
-        
+
         expand_whole_tokensequence(expanded_args[arg], begin_it, end_it,
             expand_operator_defined);
         impl::remove_placeholders(expanded_args[arg]);
         has_expanded_args[arg] = true;
     }
 }
-                    
+
 ///////////////////////////////////////////////////////////////////////////////
 // 
 //  expand_replacement_list
@@ -970,7 +973,7 @@ bool adjacent_stringize = false;
         typename ContainerT::size_type i;
 #if BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
         bool is_ellipsis = false;
-        
+
             if (IS_EXTCATEGORY((*cit), ExtParameterTokenType)) {
                 BOOST_ASSERT(boost::wave::need_variadics(ctx.get_language()));
                 i = token_id(*cit) - T_EXTPARAMETERBASE;
@@ -981,16 +984,16 @@ bool adjacent_stringize = false;
             {
                 i = token_id(*cit) - T_PARAMETERBASE;
             }
-            
+
             BOOST_ASSERT(i < arguments.size());
             if (use_replaced_arg) {
-                
+
 #if BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
                 if (is_ellipsis) {
                 position_type const &pos = (*cit).get_position();
 
                     BOOST_ASSERT(boost::wave::need_variadics(ctx.get_language()));
-                    
+
                 // ensure all variadic arguments to be expanded
                     for (typename vector<ContainerT>::size_type arg = i; 
                          arg < expanded_args.size(); ++arg)
@@ -1006,7 +1009,7 @@ bool adjacent_stringize = false;
                 // ensure argument i to be expanded
                     expand_argument(i, arguments, expanded_args, 
                         expand_operator_defined, has_expanded_args);
-                    
+
                 // replace argument
                 ContainerT const &arg = expanded_args[i];
                 
@@ -1019,7 +1022,7 @@ bool adjacent_stringize = false;
             {
             // stringize the current argument
                 BOOST_ASSERT(!arguments[i].empty());
-                
+
             // safe a copy of the first tokens position (not a reference!)
             position_type pos ((*arguments[i].begin()).get_position());
 
@@ -1073,7 +1076,7 @@ bool adjacent_stringize = false;
             "stringize ('#')", main_pos);
         return;
     }
-        
+
 // handle the cpp.concat operator
     if (seen_concat)
         concat_tokensequence(expanded);
@@ -1101,12 +1104,12 @@ macromap<ContextT>::rescan_replacement_list(token_type const &curr_token,
         if (boost::wave::need_variadics(ctx.get_language())) {
         typename ContainerT::iterator end = replacement_list.end();
         typename ContainerT::iterator it = replacement_list.begin();
-        
+
             while (it != end) {
                 using namespace boost::wave;
                 if (T_PLACEMARKER == token_id(*it)) {
                 typename ContainerT::iterator placemarker = it;
-                
+
                     ++it;
                     replacement_list.erase(placemarker);
                 }
@@ -1122,14 +1125,14 @@ macromap<ContextT>::rescan_replacement_list(token_type const &curr_token,
     on_exit::reset<bool> on_exit(macro_def.is_available_for_replacement, false);
     typename ContainerT::iterator begin_it = replacement_list.begin();
     typename ContainerT::iterator end_it = replacement_list.end();
-    
+
         expand_whole_tokensequence(expanded, begin_it, end_it, 
             expand_operator_defined);     
-        
+
     // trim replacement list, leave placeholder tokens untouched
         impl::trim_replacement_list(expanded);
     }
-    
+
     if (expanded.empty()) {
     // the resulting replacement list should contain at least a placeholder
     // token
@@ -1156,9 +1159,9 @@ macromap<ContextT>::expand_macro(ContainerT &expanded,
     defined_macros_type *scope, ContainerT *queue_symbol) 
 {
     using namespace boost::wave;
-    
+
     if (0 == scope) scope = current_macros;
-    
+
     BOOST_ASSERT(T_IDENTIFIER == token_id(curr_token) ||
         IS_CATEGORY(token_id(curr_token), KeywordTokenType) ||
         IS_EXTCATEGORY(token_id(curr_token), OperatorTokenType|AltExtTokenType) ||
@@ -1170,7 +1173,7 @@ macromap<ContextT>::expand_macro(ContainerT &expanded,
     // try to expand a predefined macro (__FILE__, __LINE__ or __INCLUDE_LEVEL__)
         if (expand_predefined_macro(curr_token, expanded))
             return false;
-        
+
     // not defined as a macro
         if (0 != queue_symbol) {
             expanded.splice(expanded.end(), *queue_symbol);
@@ -1208,8 +1211,8 @@ ContainerT replacement_list;
 
     if (T_LEFTPAREN == impl::next_token<IteratorT>::peek(first, last)) {
     // called as a function-like macro 
-        impl::skip_to_token(first, last, T_LEFTPAREN, seen_newline);
-        
+        impl::skip_to_token(ctx, first, last, T_LEFTPAREN, seen_newline);
+
 #if BOOST_WAVE_USE_DEPRECIATED_PREPROCESSING_HOOKS == 0
         IteratorT seqstart = first;
         IteratorT seqend = first;
@@ -1217,7 +1220,7 @@ ContainerT replacement_list;
 
         if (macro_def.is_functionlike) {
         // defined as a function-like macro
-        
+
         // collect the arguments
         std::vector<ContainerT> arguments;
 #if BOOST_WAVE_USE_DEPRECIATED_PREPROCESSING_HOOKS != 0
@@ -1248,7 +1251,7 @@ ContainerT replacement_list;
                 }
                 return false;
             }
-            
+
             if (count_args > macro_def.macroparameters.size() ||
                 arguments.size() > macro_def.macroparameters.size()) 
             {
@@ -1263,7 +1266,7 @@ ContainerT replacement_list;
                     return false;
                 }
             }
-                
+
         // inject tracing support
 #if BOOST_WAVE_USE_DEPRECIATED_PREPROCESSING_HOOKS != 0
             ctx.get_hooks().expanding_function_like_macro(
@@ -1276,8 +1279,9 @@ ContainerT replacement_list;
                     seqstart, seqend))
             {
                 // do not expand this macro, just copy the whole sequence 
+                expanded.push_back(curr_token);
                 std::copy(seqstart, first, 
-                    std::inserter(replacement_list, replacement_list.end()));
+                    std::inserter(expanded, expanded.end()));
                 return false;           // no further preprocessing required
             }
 #endif
@@ -1296,20 +1300,19 @@ ContainerT replacement_list;
                   macro_def.macroname, macro_def.macrodefinition, curr_token))
             {
                 // do not expand this macro, just copy the whole sequence 
-                replacement_list.push_back(curr_token);
-                ++first;                // skip macro name
+                expanded.push_back(curr_token);
                 return false;           // no further preprocessing required
             }
 #endif
 
         bool found = false;
         impl::find_concat_operator concat_tag(found);
-        
+
             std::remove_copy_if(macro_def.macrodefinition.begin(), 
                 macro_def.macrodefinition.end(), 
                 std::inserter(replacement_list, replacement_list.end()),
                 concat_tag);
-                
+
         // handle concatenation operators
             if (found && !concat_tokensequence(replacement_list))
                 return false;
@@ -1339,7 +1342,7 @@ ContainerT replacement_list;
                   macro_def.macroname, macro_def.macrodefinition, curr_token))
             {
                 // do not expand this macro, just copy the whole sequence 
-                replacement_list.push_back(curr_token);
+                expanded.push_back(curr_token);
                 ++first;                // skip macro name
                 return false;           // no further preprocessing required
             }
@@ -1347,7 +1350,7 @@ ContainerT replacement_list;
 
         bool found = false;
         impl::find_concat_operator concat_tag(found);
-        
+
             std::remove_copy_if(macro_def.macrodefinition.begin(), 
                 macro_def.macrodefinition.end(), 
                 std::inserter(replacement_list, replacement_list.end()),
@@ -1369,10 +1372,10 @@ ContainerT expanded_list;
 #else
     ctx.get_hooks().expanded_macro(ctx.derived(), replacement_list);
 #endif
-    
+
     rescan_replacement_list(curr_token, macro_def, replacement_list, 
         expanded_list, expand_operator_defined, first, last);
-    
+
 #if BOOST_WAVE_USE_DEPRECIATED_PREPROCESSING_HOOKS != 0
     ctx.get_hooks().rescanned_macro(expanded_list);  
 #else
@@ -1500,7 +1503,7 @@ macromap<ContextT>::resolve_operator_pragma(IteratorT &first,
 // isolate the parameter of the operator _Pragma
     token_type pragma_token = *first;
     
-    if (!impl::skip_to_token(first, last, T_LEFTPAREN, seen_newline)) {
+    if (!impl::skip_to_token(ctx, first, last, T_LEFTPAREN, seen_newline)) {
     // illformed operator _Pragma
         BOOST_WAVE_THROW_CTX(ctx, preprocess_exception, ill_formed_expression, 
             "operator _Pragma()", pragma_token.get_position());
@@ -1636,7 +1639,13 @@ macromap<ContextT>::is_valid_concat(string_type new_value,
         lang);
     lexer_type end = lexer_type();
     for (/**/; it != end && T_EOF != token_id(*it); ++it) 
+    {
+        // as of Wave V2.0.7 pasting of tokens is valid only if the resulting
+        // tokens are pp_tokens (as mandated by C++0x)
+        if (!is_pp_token(*it))
+            return false;
         rescanned.push_back(*it);
+    }
 
 #if BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
     if (boost::wave::need_variadics(ctx.get_language()))
@@ -1644,8 +1653,6 @@ macromap<ContextT>::is_valid_concat(string_type new_value,
 #endif 
 
 // test if the newly generated token sequence contains more than 1 token
-// the second one is the T_EOF token
-//    BOOST_ASSERT(T_EOF == token_id(rescanned.back()));
     return 1 == rescanned.size();
 }
 
@@ -1655,6 +1662,22 @@ macromap<ContextT>::is_valid_concat(string_type new_value,
 //  token sequence.
 //
 ///////////////////////////////////////////////////////////////////////////////
+template <typename Context>
+inline void report_invalid_concatenation(Context& ctx, 
+    typename Context::token_type const& prev, 
+    typename Context::token_type const& next,
+    typename Context::position_type const& main_pos)
+{
+typename Context::string_type error_string("\"");
+
+    error_string += prev.get_value();
+    error_string += "\" and \"";
+    error_string += next.get_value();
+    error_string += "\"";
+    BOOST_WAVE_THROW_CTX(ctx, preprocess_exception, invalid_concat,
+        error_string.c_str(), main_pos);
+}
+
 template <typename ContextT>
 template <typename ContainerT>
 inline bool 
@@ -1725,14 +1748,7 @@ macromap<ContextT>::concat_tokensequence(ContainerT &expanded)
                 !IS_CATEGORY(*prev, WhiteSpaceTokenType) && 
                 !IS_CATEGORY(*next, WhiteSpaceTokenType)) 
             {
-            string_type error_string("\"");
-
-                error_string += (*prev).get_value();
-                error_string += "\" and \"";
-                error_string += (*next).get_value();
-                error_string += "\"";
-                BOOST_WAVE_THROW_CTX(ctx, preprocess_exception, invalid_concat,
-                    error_string.c_str(), main_pos);
+                report_invalid_concatenation(ctx, *prev, *next, main_pos);
                 return false;
             }
 
@@ -1829,20 +1845,33 @@ position_type pos("<built-in>");
     else 
 #endif 
     {
-    // define C++ specifics
-        for (int i = 0; 0 != predef.static_data_cpp(i).name; ++i) {
-            predefined_macros::static_macros const& m = predef.static_data_cpp(i);
-            predefine_macro(current_scope, m.name, 
-                token_type(m.token_id, m.value, pos));
+#if BOOST_WAVE_SUPPORT_CPP0X != 0
+        if (boost::wave::need_cpp0x(ctx.get_language())) {
+        // define C++0x specifics
+            for (int i = 0; 0 != predef.static_data_cpp0x(i).name; ++i) {
+                predefined_macros::static_macros const& m = predef.static_data_cpp0x(i);
+                predefine_macro(current_scope, m.name, 
+                    token_type(m.token_id, m.value, pos));
+            }
         }
+        else 
+#endif 
+        {
+        // define C++ specifics
+            for (int i = 0; 0 != predef.static_data_cpp(i).name; ++i) {
+                predefined_macros::static_macros const& m = predef.static_data_cpp(i);
+                predefine_macro(current_scope, m.name, 
+                    token_type(m.token_id, m.value, pos));
+            }
 
 #if BOOST_WAVE_SUPPORT_VARIADICS_PLACEMARKERS != 0
-    // define __WAVE_HAS_VARIADICS__, if appropriate
-        if (boost::wave::need_variadics(ctx.get_language())) {
-            predefine_macro(current_scope, "__WAVE_HAS_VARIADICS__",
-                token_type(T_INTLIT, "1", pos));
-        }
+        // define __WAVE_HAS_VARIADICS__, if appropriate
+            if (boost::wave::need_variadics(ctx.get_language())) {
+                predefine_macro(current_scope, "__WAVE_HAS_VARIADICS__",
+                    token_type(T_INTLIT, "1", pos));
+            }
 #endif 
+        }
     }
 
 // predefine the __BASE_FILE__ macro which contains the main file name 
