@@ -22,7 +22,7 @@ int main()
   boost::asio::io_service io;
 
   boost::asio::deadline_timer t(io, boost::posix_time::seconds(5));
-  t.async_wait(print);
+  t.async_wait(&print);
 
   io.run();
 
