@@ -54,14 +54,14 @@ void test()
 
     win::overlapped_ptr ptr1;
 
-    win::overlapped_ptr ptr2(ios, overlapped_handler_1);
+    win::overlapped_ptr ptr2(ios, &overlapped_handler_1);
     win::overlapped_ptr ptr3(ios, overlapped_handler_2());
 
     // overlapped_ptr functions.
 
     ptr1.reset();
 
-    ptr2.reset(ios, overlapped_handler_1);
+    ptr2.reset(ios, &overlapped_handler_1);
     ptr3.reset(ios, overlapped_handler_2());
 
     OVERLAPPED* ov1 = ptr1.get();
