@@ -56,9 +56,20 @@ void test()
     HANDLE native_handle1 = INVALID_HANDLE_VALUE;
     win::stream_handle handle2(ios, native_handle1);
 
+#if defined(BOOST_ASIO_HAS_MOVE)
+    win::stream_handle handle3(std::move(handle2));
+#endif // defined(BOOST_ASIO_HAS_MOVE)
+
+    // basic_stream_handle operators.
+
+#if defined(BOOST_ASIO_HAS_MOVE)
+    handle1 = win::stream_handle(ios);
+    handle1 = std::move(handle2);
+#endif // defined(BOOST_ASIO_HAS_MOVE)
+
     // basic_io_object functions.
 
-    io_service& ios_ref = handle1.io_service();
+    io_service& ios_ref = handle1.get_io_service();
     (void)ios_ref;
 
     // basic_handle functions.
@@ -67,9 +78,9 @@ void test()
       = handle1.lowest_layer();
     (void)lowest_layer;
 
-    const win::stream_handle& handle3 = handle1;
+    const win::stream_handle& handle4 = handle1;
     const win::stream_handle::lowest_layer_type& lowest_layer2
-      = handle3.lowest_layer();
+      = handle4.lowest_layer();
     (void)lowest_layer2;
 
     HANDLE native_handle2 = INVALID_HANDLE_VALUE;
@@ -84,6 +95,10 @@ void test()
     win::stream_handle::native_type native_handle3 = handle1.native();
     (void)native_handle3;
 
+    win::stream_handle::native_handle_type native_handle4
+      = handle1.native_handle();
+    (void)native_handle4;
+
     handle1.cancel();
     handle1.cancel(ec);
 
@@ -94,13 +109,13 @@ void test()
     handle1.write_some(buffer(mutable_char_buffer), ec);
     handle1.write_some(buffer(const_char_buffer), ec);
 
-    handle1.async_write_some(buffer(mutable_char_buffer), write_some_handler);
-    handle1.async_write_some(buffer(const_char_buffer), write_some_handler);
+    handle1.async_write_some(buffer(mutable_char_buffer), &write_some_handler);
+    handle1.async_write_some(buffer(const_char_buffer), &write_some_handler);
 
     handle1.read_some(buffer(mutable_char_buffer));
     handle1.read_some(buffer(mutable_char_buffer), ec);
 
-    handle1.async_read_some(buffer(mutable_char_buffer), read_some_handler);
+    handle1.async_read_some(buffer(mutable_char_buffer), &read_some_handler);
   }
   catch (std::exception&)
   {
