@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2007. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2009. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -23,8 +23,8 @@ namespace test{
 inline void get_process_id_name(std::string &str)
 {
    std::stringstream sstr;
-   sstr << "process_" << boost::interprocess::detail::get_current_process_id();
-   str = sstr.str();
+   sstr << "process_" << boost::interprocess::ipcdetail::get_current_process_id() << std::ends;
+   str = sstr.str().c_str();
 }
 
 inline const char *get_process_id_name()
