@@ -1,7 +1,7 @@
 //////////////////////////////////////////////////////////////////////////////
 //
 // (C) Copyright Peter Dimov 2002-2005, 2007.
-// (C) Copyright Ion Gaztanaga 2006-2008.
+// (C) Copyright Ion Gaztanaga 2006-2009.
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
@@ -25,6 +25,10 @@
 #include <string>
 #include "get_process_id_name.hpp"
 
+
+#include <boost/interprocess/sync/upgradable_lock.hpp>
+#include <boost/interprocess/sync/interprocess_upgradable_mutex.hpp>
+
 using namespace boost::interprocess;
 
 class base_class
@@ -581,7 +585,7 @@ void test_alias()
          int_shared_ptr p;
          int_shared_ptr p2( p, &m );
 
-         BOOST_TEST( detail::get_pointer(p2.get()) == &m );
+         BOOST_TEST( ipcdetail::get_pointer(p2.get()) == &m );
          BOOST_TEST( p2? true: false );
          BOOST_TEST( !!p2 );
          BOOST_TEST( p2.use_count() == p.use_count() );
@@ -603,17 +607,18 @@ void test_alias()
             (shmem.construct<int>(anonymous_instance)(), shmem));
          const_int_shared_ptr p2( p, &m );
 
-         BOOST_TEST( detail::get_pointer(p2.get()) == &m );
+         BOOST_TEST( ipcdetail::get_pointer(p2.get()) == &m );
          BOOST_TEST( p2? true: false );
          BOOST_TEST( !!p2 );
          BOOST_TEST( p2.use_count() == p.use_count() );
          BOOST_TEST( !( p < p2 ) && !( p2 < p ) );
+         int_shared_ptr p_nothrow(make_managed_shared_ptr
+            (shmem.construct<int>(anonymous_instance)(), shmem, std::nothrow));
       }
    }
    shared_memory_object::remove(process_name.c_str());
 }
 
-
 int main()
 {
    if(0 != simple_test())
@@ -626,8 +631,7 @@ int main()
       return 1;
 
    test_alias();
-
-   return 0;
 }
 
 #include <boost/interprocess/detail/config_end.hpp>
+
