@@ -1,7 +1,7 @@
 /////////////////////////////////////////////////////////////////////////////
 //
 // (C) Copyright Olaf Krzikalla 2004-2006.
-// (C) Copyright Ion Gaztanaga  2006-2008.
+// (C) Copyright Ion Gaztanaga  2006-2009.
 //
 // Distributed under the Boost Software License, Version 1.0.
 //    (See accompanying file LICENSE_1_0.txt or copy at
@@ -23,6 +23,26 @@
 #include "test_macros.hpp"
 #include "test_container.hpp"
 
+namespace boost { namespace intrusive { namespace test {
+
+#if !defined (BOOST_INTRUSIVE_VARIADIC_TEMPLATES)
+template<class T, class O1, class O2, class O3, class O4, class O5, class O6>
+#else
+template<class T, class ...Options>
+#endif
+struct is_unordered<boost::intrusive::unordered_multiset<T,
+   #if !defined (BOOST_INTRUSIVE_VARIADIC_TEMPLATES)
+   O1, O2, O3, O4, O5, O6
+   #else
+   Options...
+   #endif
+> >
+{
+   static const bool value = true;
+};
+
+}}}
+
 using namespace boost::intrusive;
 
 struct my_tag;
@@ -719,7 +739,6 @@ class test_main_template
                 , false
                 , Incremental
                 >::test_all(data);
-
       return 0;
    }
 };
@@ -740,7 +759,7 @@ class test_main_template<VoidPointer, false, Incremental>
                   < value_type
                   , typename hooks<VoidPointer>::base_hook_type
                   >::type
-                , true
+                , false
                 , false
                 , Incremental
                 >::test_all(data);
@@ -752,7 +771,7 @@ class test_main_template<VoidPointer, false, Incremental>
                                , &value_type::node_
                                >
                   >::type
-                , false
+                , true
                 , false
                 , Incremental
                 >::test_all(data);
@@ -761,8 +780,8 @@ class test_main_template<VoidPointer, false, Incremental>
                   < value_type
                   , typename hooks<VoidPointer>::auto_base_hook_type
                   >::type
-                , true
-                , true
+                , false
+                , false
                 , Incremental
                 >::test_all(data);
 
