@@ -28,6 +28,7 @@ typedef long double comparison_type;
 
 template <class T> void sink(T const&) {}
 template <class T> T rvalue(T const& v) { return v; }
+template <class T> T rvalue_default() { return T(); }
 
 template <class X, class T>
 void container_test(X& r, T const&)
@@ -109,30 +110,15 @@ void container_test(X& r, T const&)
     BOOST_TEST(X().size() == 0);
 
     X a,b;
+    X a_const;
 
     sink(X(a));
     X u2(a);
     X u3 = a;
 
-    X* ptr = new X();
-    X& a1 = *ptr;
-    (&a1)->~X();
-
-    X const a_const;
-    test::check_return_type<iterator>::equals(a.begin());
-    test::check_return_type<const_iterator>::equals(a_const.begin());
-    test::check_return_type<const_iterator>::equals(a.cbegin());
-    test::check_return_type<const_iterator>::equals(a_const.cbegin());
-    test::check_return_type<iterator>::equals(a.end());
-    test::check_return_type<const_iterator>::equals(a_const.end());
-    test::check_return_type<const_iterator>::equals(a.cend());
-    test::check_return_type<const_iterator>::equals(a_const.cend());
-
     a.swap(b);
+    boost::swap(a, b);
     test::check_return_type<X>::equals_ref(r = a);
-    test::check_return_type<size_type>::equals(a.size());
-    test::check_return_type<size_type>::equals(a.max_size());
-    test::check_return_type<bool>::convertible(a.empty());
 
     // Allocator
 
@@ -146,6 +132,51 @@ void container_test(X& r, T const&)
     sink(u3);
 }
 
+template <class X>
+void unordered_destructible_test(X&)
+{
+    typedef BOOST_DEDUCED_TYPENAME X::iterator iterator;
+    typedef BOOST_DEDUCED_TYPENAME X::const_iterator const_iterator;
+    typedef BOOST_DEDUCED_TYPENAME X::size_type size_type;
+
+    X x1;
+
+#if !defined(BOOST_NO_RVALUE_REFERENCES)
+    X x2(rvalue_default<X>());
+    X x3 = rvalue_default<X>();
+    // This can only be done if propagate_on_container_move_assignment::value
+    // is true.
+    // x2 = rvalue_default<X>();
+#endif
+
+    X* ptr = new X();
+    X& a1 = *ptr;
+    (&a1)->~X();
+
+    X a,b;
+    X const a_const;
+    test::check_return_type<iterator>::equals(a.begin());
+    test::check_return_type<const_iterator>::equals(a_const.begin());
+    test::check_return_type<const_iterator>::equals(a.cbegin());
+    test::check_return_type<const_iterator>::equals(a_const.cbegin());
+    test::check_return_type<iterator>::equals(a.end());
+    test::check_return_type<const_iterator>::equals(a_const.end());
+    test::check_return_type<const_iterator>::equals(a.cend());
+    test::check_return_type<const_iterator>::equals(a_const.cend());
+
+    a.swap(b);
+    boost::swap(a, b);
+ 
+    test::check_return_type<size_type>::equals(a.size());
+    test::check_return_type<size_type>::equals(a.max_size());
+    test::check_return_type<bool>::convertible(a.empty());
+
+    // Allocator
+
+    typedef BOOST_DEDUCED_TYPENAME X::allocator_type allocator_type;
+    test::check_return_type<allocator_type>::equals(a_const.get_allocator());
+}
+
 template <class X, class Key>
 void unordered_set_test(X&, Key const&)
 {
@@ -160,6 +191,7 @@ void unordered_map_test(X& r, Key const& k, T const& v)
 {
     typedef BOOST_DEDUCED_TYPENAME X::value_type value_type;
     typedef BOOST_DEDUCED_TYPENAME X::key_type key_type;
+
     BOOST_MPL_ASSERT((
         boost::is_same<value_type, std::pair<key_type const, T> >));
 
@@ -180,6 +212,8 @@ void equality_test(X& r)
 
     test::check_return_type<bool>::equals(a == b);
     test::check_return_type<bool>::equals(a != b);
+    test::check_return_type<bool>::equals(boost::operator==(a, b));
+    test::check_return_type<bool>::equals(boost::operator!=(a, b));
 }
 
 template <class X, class T>
@@ -211,9 +245,11 @@ void unordered_map_functions(X&, Key const& k, T const&)
     test::check_return_type<mapped_type const>::equals_ref(b.at(k));
 }
 
-template <class X, class Key, class T, class Hash, class Pred>
-void unordered_test(X&, Key& k, T& t, Hash& hf, Pred& eq)
+template <class X, class Key, class Hash, class Pred>
+void unordered_test(X& x, Key& k, Hash& hf, Pred& eq)
 {
+    unordered_destructible_test(x);
+
     typedef BOOST_DEDUCED_TYPENAME X::key_type key_type;
     typedef BOOST_DEDUCED_TYPENAME X::hasher hasher;
     typedef BOOST_DEDUCED_TYPENAME X::key_equal key_equal;
@@ -277,8 +313,8 @@ void unordered_test(X&, Key& k, T& t, Hash& hf, Pred& eq)
         const_local_iterator_reference;
 
     BOOST_MPL_ASSERT((boost::is_same<Key, key_type>));
-    boost::function_requires<boost::CopyConstructibleConcept<key_type> >();
-    boost::function_requires<boost::AssignableConcept<key_type> >();
+    //boost::function_requires<boost::CopyConstructibleConcept<key_type> >();
+    //boost::function_requires<boost::AssignableConcept<key_type> >();
 
     BOOST_MPL_ASSERT((boost::is_same<Hash, hasher>));
     test::check_return_type<std::size_t>::equals(hf(k));
@@ -316,45 +352,18 @@ void unordered_test(X&, Key& k, T& t, Hash& hf, Pred& eq)
     X();
     X a4;
 
-    BOOST_DEDUCED_TYPENAME X::value_type* i = 0;
-    BOOST_DEDUCED_TYPENAME X::value_type* j = 0;
-
-    X(i, j, 10, hf, eq);
-    X a5(i, j, 10, hf, eq);
-    X(i, j, 10, hf);
-    X a6(i, j, 10, hf);
-    X(i, j, 10);
-    X a7(i, j, 10);
-    X(i, j);
-    X a8(i, j);
-
-    X const b;
-    sink(X(b));
-    X a9(b);
-    a = b;
-
-    test::check_return_type<hasher>::equals(b.hash_function());
-    test::check_return_type<key_equal>::equals(b.key_eq());
-
-    const_iterator q = a.cbegin();
-    test::check_return_type<iterator>::equals(a.insert(q, t));
-    test::check_return_type<iterator>::equals(a.emplace_hint(q, t));
-
-    a.insert(i, j);
     test::check_return_type<size_type>::equals(a.erase(k));
 
-    BOOST_TEST(a.empty());
-    if(a.empty()) {
-        a.insert(t);
-        q = a.cbegin();
-        test::check_return_type<iterator>::equals(a.erase(q));
-    }
-
     const_iterator q1 = a.cbegin(), q2 = a.cend();
     test::check_return_type<iterator>::equals(a.erase(q1, q2));
 
     a.clear();
 
+    X const b;
+
+    test::check_return_type<hasher>::equals(b.hash_function());
+    test::check_return_type<key_equal>::equals(b.key_eq());
+
     test::check_return_type<iterator>::equals(a.find(k));
     test::check_return_type<const_iterator>::equals(b.find(k));
     test::check_return_type<size_type>::equals(b.count(k));
@@ -388,9 +397,118 @@ void unordered_test(X&, Key& k, T& t, Hash& hf, Pred& eq)
     sink(a2);
     sink(a3);
     sink(a4);
+}
+
+template <class X, class Key, class T, class Hash, class Pred>
+void unordered_copyable_test(X& x, Key& k, T& t, Hash& hf, Pred& eq)
+{
+    unordered_test(x, k, hf, eq);
+
+    typedef BOOST_DEDUCED_TYPENAME X::iterator iterator;
+    typedef BOOST_DEDUCED_TYPENAME X::const_iterator const_iterator;
+
+    X a;
+
+    BOOST_DEDUCED_TYPENAME X::value_type* i = 0;
+    BOOST_DEDUCED_TYPENAME X::value_type* j = 0;
+
+    X(i, j, 10, hf, eq);
+
+    X a5(i, j, 10, hf, eq);
+    X(i, j, 10, hf);
+    X a6(i, j, 10, hf);
+    X(i, j, 10);
+    X a7(i, j, 10);
+    X(i, j);
+    X a8(i, j);
+
+    X const b;
+    sink(X(b));
+    X a9(b);
+    a = b;
+
+    const_iterator q = a.cbegin();
+
+    test::check_return_type<iterator>::equals(a.insert(q, t));
+    test::check_return_type<iterator>::equals(a.emplace_hint(q, t));
+
+    a.insert(i, j);
+
+    X a10;
+    a10.insert(t);
+    q = a10.cbegin();
+    test::check_return_type<iterator>::equals(a10.erase(q));
+
+    // Avoid unused variable warnings:
+
+    sink(a);
     sink(a5);
     sink(a6);
     sink(a7);
     sink(a8);
     sink(a9);
 }
+
+template <class X, class Key, class T, class Hash, class Pred>
+void unordered_movable_test(X& x, Key& k, T& /* t */, Hash& hf, Pred& eq)
+{
+    unordered_test(x, k, hf, eq);
+
+    typedef BOOST_DEDUCED_TYPENAME X::iterator iterator;
+    typedef BOOST_DEDUCED_TYPENAME X::const_iterator const_iterator;
+
+#if !defined(BOOST_NO_RVALUE_REFERENCES)
+    X x1(rvalue_default<X>());
+    X x2(boost::move(x1));
+    x1 = rvalue_default<X>();
+    x2 = boost::move(x1);
+#endif
+
+    test::minimal::constructor_param* i = 0; 
+    test::minimal::constructor_param* j = 0;
+
+    X(i, j, 10, hf, eq);
+    X a5(i, j, 10, hf, eq);
+    X(i, j, 10, hf);
+    X a6(i, j, 10, hf);
+    X(i, j, 10);
+    X a7(i, j, 10);
+    X(i, j);
+    X a8(i, j);
+
+    X a;
+
+    const_iterator q = a.cbegin();
+
+    test::minimal::constructor_param v;
+    a.emplace(v);
+    test::check_return_type<iterator>::equals(a.emplace_hint(q, v));
+
+    T v1(v);
+    a.emplace(boost::move(v1));
+    T v2(v);
+    a.insert(boost::move(v2));
+    T v3(v);
+    test::check_return_type<iterator>::equals(
+            a.emplace_hint(q, boost::move(v3)));
+    T v4(v);
+    test::check_return_type<iterator>::equals(
+            a.insert(q, boost::move(v4)));
+
+    a.insert(i, j);
+
+    X a10;
+    T v5(v);
+    a10.insert(boost::move(v5));
+    q = a10.cbegin();
+    test::check_return_type<iterator>::equals(a10.erase(q));
+
+    // Avoid unused variable warnings:
+
+    sink(a);
+    sink(a5);
+    sink(a6);
+    sink(a7);
+    sink(a8);
+    sink(a10);
+}
