@@ -5,7 +5,7 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2008 Hartmut Kaiser. Distributed under the Boost 
+    Copyright (c) 2001-2011 Hartmut Kaiser. Distributed under the Boost 
     Software License, Version 1.0. (See accompanying file 
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
@@ -57,18 +57,18 @@ public:
 #endif // 0 != __DECCXX_VER
     {}
 
-// interface to the boost::spirit::classic::multi_pass_policies::functor_input 
+// interface to the boost::spirit::classic::iterator_policies::functor_input 
 // policy
     typedef TokenT result_type;
     typedef slex_iterator_functor_shim unique;
     typedef lex_input_interface<TokenT>* shared;
 
     BOOST_WAVE_EOF_PREFIX result_type const eof;
-    
+
     template <typename MultiPass>
     static result_type& get_next(MultiPass& mp, result_type& result)
     { 
-        return mp.shared->ftor->get(result); 
+        return mp.shared()->ftor->get(result); 
     }
 
     // this will be called whenever the last reference to a multi_pass will
@@ -76,22 +76,22 @@ public:
     template <typename MultiPass>
     static void destroy(MultiPass& mp)
     { 
-        delete mp.shared->ftor; 
+        delete mp.shared()->ftor; 
     }
 
     template <typename MultiPass>
     static void set_position(MultiPass& mp, position_type const &pos)
     {
-        mp.shared->ftor->set_position(pos);
+        mp.shared()->ftor->set_position(pos);
     }
-    
+
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
     template <typename MultiPass>
     static bool has_include_guards(MultiPass& mp, std::string& guard_name) 
     {
-        return mp.shared->ftor->has_include_guards(guard_name);
+        return mp.shared()->ftor->has_include_guards(guard_name);
     }
-#endif    
+#endif
 
 private:
     boost::shared_ptr<lex_input_interface<TokenT> > functor_ptr;
@@ -141,16 +141,16 @@ struct make_multi_pass
     functor_data_type;
     typedef typename FunctorData::result_type result_type;
 
-    typedef boost::spirit::multi_pass_policies::split_functor_input input_policy;
-    typedef boost::spirit::multi_pass_policies::ref_counted ownership_policy;
+    typedef boost::spirit::iterator_policies::split_functor_input input_policy;
+    typedef boost::spirit::iterator_policies::ref_counted ownership_policy;
 #if defined(BOOST_WAVE_DEBUG)
-    typedef boost::spirit::multi_pass_policies::buf_id_check check_policy;
+    typedef boost::spirit::iterator_policies::buf_id_check check_policy;
 #else
-    typedef boost::spirit::multi_pass_policies::no_check check_policy;
+    typedef boost::spirit::iterator_policies::no_check check_policy;
 #endif
-    typedef boost::spirit::multi_pass_policies::split_std_deque storage_policy;
-    
-    typedef boost::spirit::multi_pass_policies::default_policy<
+    typedef boost::spirit::iterator_policies::split_std_deque storage_policy;
+
+    typedef boost::spirit::iterator_policies::default_policy<
             ownership_policy, check_policy, input_policy, storage_policy>
         policy_type;
     typedef boost::spirit::multi_pass<functor_data_type, policy_type> type;
@@ -169,13 +169,13 @@ class slex_iterator
 
     typedef typename input_policy_type::unique unique_functor_type;
     typedef typename input_policy_type::shared shared_functor_type;
-    
+
 public:
     typedef TokenT token_type;
-    
+
     slex_iterator()
     {}
-    
+
     template <typename IteratorT>
     slex_iterator(IteratorT const &first, IteratorT const &last, 
             typename TokenT::position_type const &pos, 
@@ -192,22 +192,22 @@ public:
     void set_position(typename TokenT::position_type const &pos)
     {
         typedef typename token_type::position_type position_type;
-        
+
     // set the new position in the current token
-    token_type const& currtoken = base_type::get_input();
+    token_type const& currtoken = this->base_type::dereference(*this);
     position_type currpos = currtoken.get_position();
-    
+
         currpos.set_file(pos.get_file());
         currpos.set_line(pos.get_line());
-        base_type::get_input().set_position(currpos);
-        
+        const_cast<token_type&>(currtoken).set_position(currpos);
+
     // set the new position for future tokens as well
         if (token_type::string_type::npos != 
             currtoken.get_value().find_first_of('\n'))
         {
             currpos.set_line(pos.get_line() + 1);
         }
-        unique_functor_type::get_functor().set_position(currpos);
+        unique_functor_type::set_position(*this, currpos);
     }
 
 #if BOOST_WAVE_SUPPORT_PRAGMA_ONCE != 0
@@ -216,9 +216,9 @@ public:
     // completely
     bool has_include_guards(std::string& guard_name) const
     {
-        return unique_functor_type::has_include_guards(guard_name);
+        return unique_functor_type::has_include_guards(*this, guard_name);
     }
-#endif    
+#endif
 };
 
 ///////////////////////////////////////////////////////////////////////////////
